[+:   -*- Mode: nroff -*-

AutoGen5 template man=%s.1

## Copyright (C) 2001-2018 Bruce Korb, all rights reserved.

:+]
[+: `echo please note that this is obsolete >&2` :+][+:

INCLUDE "agman-cmd.tpl"

\:+]
